#include "CdvaImpl.h"

#include <sstream>
#include <opencv2/opencv.hpp>
#include <algorithm>

// include here other classes if needed

using namespace cv;
using namespace mpeg7cdva;
using namespace std;


CdvaImpl::CdvaImpl():verboseMode(false)
{
	// TODO constructor
}

CdvaImpl::~CdvaImpl()
{
	// TODO destructor
}

void CdvaImpl::init(bool verbose, size_t n_videos, int querybitrate, int refbitrate)
{
	verboseMode = verbose;
	srand((unsigned int)(n_videos + querybitrate + refbitrate));		// random seed

	// TODO initialize - implement global initialization here
}

void CdvaImpl::close()
{
	// TODO de-initialize - cleanup data here, close files, etc.
}

/*
 * Your CDVA descriptor extension; may be changed (if needed) to identify a specific implementation.
 */
const char * CdvaImpl::getDescriptorExt(int bitrate)
{
	switch (bitrate)
	{
	case   0: return ".DB.cdva";
	case  16: return ".16K.cdva";
	case  64: return ".64K.cdva";
	case 256: return ".256K.cdva";
	}
	return NULL;
}

/*
 * Check valid bitrates
 */
bool CdvaImpl::checkBitrate(int bitrate)
{
	switch (bitrate)
	{
	case   0: return true;
	case  16: return true;
	case  64: return true;
	case 256: return true;
	}
	return false;
}

/*
 * CDVA extract implementation: can be changed freely;
 * you can signal errors at any point of your code using "throw CdvaException(string);"
 */
void CdvaImpl::extract(const std::string & descrname, const std::string & videopathname, int mode, LogManager & logmanager)
{
	VideoCapture videocpp(videopathname);

	if(!videocpp.isOpened())
	{
		throw CdvaException(string("File not found: ").append(videopathname));
	}

	double tot_frames = videocpp.get(CV_CAP_PROP_FRAME_COUNT);	// get the total number of frames of the video

	logmanager.setNumFrames(tot_frames);

	int base_frame = 0;							// base frame counter in the current shot
	int current_frame = 0;

	// MAIN LOOP: encode all frames using CDVA

	ostringstream osserr;

	Mat colorimage;				// input image
	Mat greyimage;				// converted greyscale image

	for (int k = 0; k < tot_frames; ++k)
	{
		videocpp.read(colorimage);
		cvtColor(colorimage, greyimage, cv::COLOR_BGR2GRAY);		// convert to luminance only

		// TODO encode keypoints using greyimage.data or colorimage.data: e.g.: encode(output, greyimage.cols, greyimage.rows, greyimage.data);

		if (verboseMode)
		{
			// TODO replace the following with your own extra information
			cv::imshow(videopathname, colorimage);
			waitKey(40);
		}

	}		// end for

	// TODO replace random value with the real descriptor size (in bytes)
	logmanager.setDescriptorLength(3000 * (double) rand()/ (double) RAND_MAX);

	std::ofstream fout;
	fout.open(descrname.c_str());		// open output descriptor (TODO: change it to binary mode = fout.open(descrname, std::ofstream::binary))
	fout << "test using mode " << mode << endl;
	fout.close();

	double time_s = 0.001 * videocpp.get(CV_CAP_PROP_POS_MSEC);
	logmanager.setVideoDuration(time_s);
}

/*
 * CDVA match implementation: can be changed freely;
 * you can signal errors at any point of your code using "throw CdvaException(string);"
 */
double CdvaImpl::match(MatchData & matchResults, const std::string & qdescrname, const std::string & rdescrname, int qmode, int rmode)
{
	double score = 0;
	ifstream qreader(qdescrname.c_str(), std::ifstream::binary);
	ifstream rreader(rdescrname.c_str(), std::ifstream::binary);

	if (! qreader.good())
		throw CdvaException(string("Cannot read descriptor: ").append(qdescrname));

	if (! rreader.good())
		throw CdvaException(string("Cannot read descriptor: ").append(rdescrname));

	// TODO replace the following instructions with actual code

	for (int k=0; k<3; ++k)
	{
		double randomTime = 16.0 * (double) rand() / (double) RAND_MAX ;
		matchResults.setMatchingTime(randomTime);
	}

	score = (double) rand() / (double) RAND_MAX ;
	matchResults.setMatchingScore(score);

	return score;
}


/*
 * CDVA makeindex implementation: can be changed freely;
 * you can signal errors at any point of your code using "throw CdvaException(string);"
 * the following default implementation of makeindex just appends id + descriptor data to the index file.
 */
void  CdvaImpl::makeindex(const std::string & cdva_descriptor, const std::string & relativepathname)
{
	// read descriptor and append it to the DB
	DbRecord record;
	record.descriptorId = relativepathname;
	record.descriptor.read(cdva_descriptor.c_str());	// read descriptor from file
	DB.push_back(record);			// save record in memory
}

//
// order MatchData elements by descending score
//
bool CdvaImpl::byDescendingScore(const MatchData & m1, const MatchData & m2) {
   return (m1.getScore() > m2.getScore());
 }

/*
 * CDVA retrieve implementation: can be changed freely;
 * you can signal errors at any point of your code using "throw CdvaException(string);"
 */
void CdvaImpl::retrieve(vector<MatchData> & retrievalResults, const std::string & qdescrname, int qmode,  LogManager & logmanager)
{
	ifstream qreader(qdescrname.c_str(), std::ifstream::binary);

	if (! qreader.good())
		throw CdvaException(string("Cannot read descriptor: ").append(qdescrname));

	// TODO replace the following instruction with actual code

	int numResults = min((int) DB.size(), 50);

	for (int i=0; i<numResults; ++i)		// for all elements in the index Data Base...
	{
		MatchData results;

		for (int k=0; k<3; ++k)
		{
			double randomTime = 16.0 * (double) rand() / (double) RAND_MAX ;
			results.setMatchingTime(randomTime);
		}

		results.setMatchingScore((double) rand() / (double) RAND_MAX );
		results.setReferenceID(DB[i].descriptorId);

		retrievalResults.push_back(results);	// store results
	}

	// sort results by descending score
	sort(retrievalResults.begin(), retrievalResults.end(), byDescendingScore);
}
