
#pragma once

#include <fstream>
#include <string>
#include <vector>
#include "CdvaException.h"
#include "LogManager.h"
#include "Buffer.h"

/**
 * @namespace mpeg7cdva
 * Namespace used to encapsulate all MPEG-7 CDVA declarations that are visible when
 * the CDVA headers (in particular cdva.h) are included.
 */
namespace mpeg7cdva
{
// put here your other class declarations


/**
 * @class DbRecord
 * a container class for DB records
 */
class DbRecord
{
public:
	std::string descriptorId;	///< the string that identifies the video (e.g. pathname)
	Buffer descriptor;			///< the buffer containing the video descriptor
};

/**
 * @class CdvaImpl
 * A CDVA implementation template class.
 */
class CdvaImpl
{
private:
	bool verboseMode;			// how much information to show
	std::vector<DbRecord> DB;	// the index Data Base - may be replaced by a real DB

	static bool byDescendingScore(const MatchData & m1, const MatchData & m2);	// order data by descending score

public:
	/**
	 * check if the given bitrate is one of the standard values defined in the CDVA evaluation framework.
	 * @param bitrate the bitrate in Kilo-byte per second (KB/s)
	 * @return true if valid
	 */
	static bool checkBitrate(int bitrate);

	/**
	 * get the file extension corresponding to the given bitrate.
	 * @param bitrate the bitrate in Kilo-byte per second (KB/s)
	 * @return the file extension
	 */
	static const char * getDescriptorExt(int bitrate);

	CdvaImpl();
	virtual ~CdvaImpl();

	/**
	 * initialization method - called once before processing videos.
	 * @param verbose when set, more information is provided
	 * @param n_videos the number of videos to be processed
	 * @param querybitrate the query encoding bitrate (one of 0,16,64,256)
	 * @param refbitrate the reference encoding bitrate (one of 0,16,64,256)
	 */
	void init(bool verbose, size_t n_videos, int querybitrate, int refbitrate = 0);

	/**
	 * Video processing method - called once for each video in the list.
	 * @param descrname output descriptor pathname
	 * @param videopathname input video stream pathname
	 * @param bitrate encoding bitrate (one of 0,16,64,256)
	 * @param logmanager the LogManager instance that saves data into log files
	 */
	void extract(const std::string & descrname, const std::string & videopathname, int bitrate, LogManager & logmanager);

	/**
	 * Video matching method -  called once for each pair of videos in the list.
	 * @param matchResults container for the results of matching
	 * @param qdescrname input query descriptor name
	 * @param rdescrname input reference descriptor name
	 * @param qbitrate query bitrate (one of 16,64,256)
	 * @param rbitrate reference bitrate (one of 16,64,256)
	 * @return the matching score (normalized in the [0..1] range)
	 */
	double match(MatchData & matchResults, const std::string & qdescrname, const std::string & rdescrname, int qbitrate, int rbitrate);

	/**
	 * Video indexing method - builds a DB of reference video descriptors.
	 * @param cdva_descriptor the descriptor to add to the DB
	 * @param relativepathname the relative pathname of the video file to be used as unique identifier
	 */
	void makeindex(const std::string & cdva_descriptor, const std::string & relativepathname);

	/**
	 * Video retrieval method - returns a list of reference videos matching the query video.
	 * @param retrievalResults the output vector containing an ordered list of matching reference videos
	 * @param qdescrname the video query descriptor
	 * @param qbitrate query bitrate (one of 16,64,256)
	 * @param logmanager the LogManager instance that saves data into log files
	 */
	void retrieve(std::vector<MatchData> & retrievalResults, const std::string & qdescrname, int qbitrate,  LogManager & logmanager);

	/**
	 * de-initialization method - called once at the end of processing.
	 */
	void close();

};	// end class CDVAImpl

}  // end of namespace mpeg7cdva
