#pragma once
#include <string>

/**
@mainpage Documentation

This is the documentation of the C++ classes implementing the MPEG CDVA Evaluation Framework.
The software implements the recommendations contained in the following document:
	- N15338:"Evaluation Framework for Compact Descriptors for Video Analysis - Search and Retrieval", July 2015, Warsaw, Poland

Documentation on how to build and install the code is contained in the "CDVA_build_run_instructions" document which can be found in the "docs" directory.
*/



/**
 * @namespace mpeg7cdva
 * Namespace used to encapsulate all MPEG-7 CDVA declarations that are visible when
 * the CDVA Library headers (in particular CdvaInterface.h) are included.
 */
namespace mpeg7cdva
{

/**
 * @class MatchData
 * A class containing the results of a matching or retrieval operation.
 */
class MatchData {
private:
	double score;					// score of matching
	std::string reference_ID;		// identifier of the matching reference video
	double startTime;				// time in seconds indicating the fist matching frame of the query clip (e.g. 3.456 s)
	double endTime;					// time in seconds indicating the last matching frame of the query clip (e.g. 6.352 s)

public:
	MatchData()
	{
		score = 0;
		startTime = endTime = -1;
	}

	virtual ~MatchData()
	{}


	/**
	 * set the score of matching the query image with the reference image.
	 * @param myscore the overall matching score
	 */
	void setMatchingScore(double myscore)
	{
		score = myscore;		// save score
	}

	/**
	 * set the time of each frame matching (only the first and the last will be saved).
	 * @param time_s the time in seconds from the start of the query video sequence
	 */
	void setMatchingTime(double time_s)
	{
		if (startTime < 0)
		{
			startTime = endTime = time_s;		// this is the first frame matching
		}
		else
		{
			if (time_s > endTime)
				endTime = time_s;

			if (time_s < startTime)
				startTime = time_s;
		}
	}

	/**
	 * set the string that identifies the matching reference video clip.
	 * @param reference the identifier (usually the relative pathname) of the matching reference video.
	 */
	void setReferenceID(const std::string reference)
	{
		reference_ID = reference;
	}

	/**
	 * Get the matching score.
	 * @return the score
	 */
	double getScore() const
	{
		return score;
	}

	/**
	 * get the time in seconds indicating the fist matching frame of the query clip.
	 * @return the time in seconds from the start of the video clip
	 */
	double getFirstMatchingTime() const
	{
		return startTime;
	}

	/**
	 * get the time in seconds indicating the last matching frame of the query clip.
	 * @return the time in seconds from the start of the video clip
	 */
	double getLastMatchingTime() const
	{
		return endTime;
	}

	/**
	 * get the string that identifies the matching reference video clip.
	 * @return the video clip relative pathname
	 */
	std::string getReferenceId() const
	{
		return reference_ID;
	}

};

}	// end of namespace
